@@ -20,6 +20,9 @@ import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.io.Reader;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -30,11 +33,12 @@ import java.util.StringTokenizer;
 import java.util.Vector;
 
 import org.codehaus.plexus.util.Os;
+import org.codehaus.plexus.util.ReaderFactory;
 import org.codehaus.plexus.util.StringUtils;
 
 /**
  * @author <a href="mailto:trygvis@inamo.no">Trygve Laugst&oslash;l </a>
- * @version $Id: CommandLineUtils.java 8253 2009-06-05 00:18:27Z igorife $
+ * @version $Id: CommandLineUtils.java 8700 2010-04-24 12:03:28Z bentmann $
  */
 public abstract class CommandLineUtils
 {
@@ -118,12 +122,12 @@ public abstract class CommandLineUtils
     }
 
     /**
-     * @param cl
-     * @param systemIn
-     * @param systemOut
-     * @param systemErr
-     * @param timeoutInSeconds
-     * @return
+     * @param cl  The command line to execute
+     * @param systemIn  The input to read from, must be thread safe
+     * @param systemOut  A consumer that receives output, must be thread safe
+     * @param systemErr  A consumer that receives system error stream output, must be thread safe
+     * @param timeoutInSeconds Positive integer to specify timeout, zero and negative integers for no timeout.
+     * @return A return value, see {@link Process#exitValue()}
      * @throws CommandLineException or CommandLineTimeOutException if time out occurs
      */
     public static int executeCommandLine( Commandline cl, InputStream systemIn, StreamConsumer systemOut,
@@ -213,6 +217,16 @@ public abstract class CommandLineUtils
 
             processes.remove( new Long( cl.getPid() ) );
 
+            if ( outputPumper.getException() != null )
+            {
+                throw new CommandLineException( "Error inside systemOut parser", outputPumper.getException() );
+            }
+
+            if ( errorPumper.getException() != null )
+            {
+                throw new CommandLineException( "Error inside systemErr parser", errorPumper.getException() );
+            }
+
             return returnValue;
         }
         catch ( InterruptedException ex )
@@ -248,6 +262,8 @@ public abstract class CommandLineUtils
      *
      * @return The shell environment variables, can be empty but never <code>null</code>.
      * @throws IOException If the environment variables could not be queried from the shell.
+     * @see System#getenv() System.getenv() API, new in JDK 5.0, to get the same result
+     * <b>since 2.0.2 System#getenv() will be used if available in the current running jvm.</b>
      */
     public static Properties getSystemEnvVars()
         throws IOException
@@ -262,10 +278,36 @@ public abstract class CommandLineUtils
      * @param caseSensitive Whether environment variable keys should be treated case-sensitively.
      * @return Properties object of (possibly modified) envar keys mapped to their values.
      * @throws IOException
+     * @see System#getenv() System.getenv() API, new in JDK 5.0, to get the same result
+     * <b>since 2.0.2 System#getenv() will be used if available in the current running jvm.</b>
      */
     public static Properties getSystemEnvVars( boolean caseSensitive )
         throws IOException
     {
+        
+        // check if it's 1.5+ run 
+        
+        Method getenvMethod = getEnvMethod();
+        if ( getenvMethod != null )
+        {
+            try
+            {
+                return getEnvFromSystem( getenvMethod, caseSensitive );
+            }
+            catch ( IllegalAccessException e )
+            {
+                throw new IOException( e.getMessage() );
+            }
+            catch ( IllegalArgumentException e )
+            {
+                throw new IOException( e.getMessage() );
+            }
+            catch ( InvocationTargetException e )
+            {
+                throw new IOException( e.getMessage() );
+            }
+        }
+        
         Process p = null;
 
         try
@@ -275,6 +317,7 @@ public abstract class CommandLineUtils
             Runtime r = Runtime.getRuntime();
 
             //If this is windows set the shell to command.com or cmd.exe with correct arguments.
+            boolean overriddenEncoding = false;
             if ( Os.isFamily( Os.FAMILY_WINDOWS ) )
             {
                 if ( Os.isFamily( Os.FAMILY_WIN9X ) )
@@ -283,7 +326,10 @@ public abstract class CommandLineUtils
                 }
                 else
                 {
-                    p = r.exec( "cmd.exe /c set" );
+                    overriddenEncoding = true;
+                    // /U = change stdout encoding to UTF-16LE to avoid encoding inconsistency
+                    // between command-line/DOS and GUI/Windows, see PLXUTILS-124
+                    p = r.exec( "cmd.exe /U /c set" );
                 }
             }
             else
@@ -291,7 +337,10 @@ public abstract class CommandLineUtils
                 p = r.exec( "env" );
             }
 
-            BufferedReader br = new BufferedReader( new InputStreamReader( p.getInputStream() ) );
+            Reader reader = overriddenEncoding
+                ? new InputStreamReader( p.getInputStream(), ReaderFactory.UTF_16LE )
+                : new InputStreamReader( p.getInputStream() );
+            BufferedReader br = new BufferedReader( reader );
 
             String line;
 
@@ -578,5 +627,39 @@ public abstract class CommandLineUtils
         }
         return result.toString();
     }
-
+    
+    private static Method getEnvMethod()
+    {
+        try
+        {
+            return System.class.getMethod( "getenv", null );
+        }
+        catch ( NoSuchMethodException e )
+        {
+            return null;
+        }
+        catch ( SecurityException e )
+        {
+            return null;
+        }
+    }
+    
+    private static Properties getEnvFromSystem( Method method, boolean caseSensitive )
+        throws IllegalAccessException, IllegalArgumentException, InvocationTargetException
+    {
+        Properties envVars = new Properties();
+        Map envs = (Map) method.invoke( null, null );
+        Iterator iterator = envs.keySet().iterator();
+        while ( iterator.hasNext() )
+        {
+            String key = (String) iterator.next();
+            String value = (String) envs.get( key );
+            if ( !caseSensitive )
+            {
+                key = key.toUpperCase( Locale.ENGLISH );
+            }
+            envVars.put( key, value );
+        }
+        return envVars;
+    }
 }
