@@ -21,12 +21,14 @@ import org.codehaus.plexus.util.xml.pull.XmlSerializer;
 import java.io.IOException;
 import java.io.StringWriter;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
 /**
- * @version $Id: Xpp3Dom.java 8215 2009-05-24 16:32:26Z bentmann $
+ * @version $Id: Xpp3Dom.java 8680 2010-04-06 20:05:21Z bentmann $
  * NOTE: remove all the util code in here when separated, this class should be pure data.
  */
 public class Xpp3Dom
@@ -43,6 +45,8 @@ public class Xpp3Dom
 
     protected Xpp3Dom parent;
 
+    private static final String[] EMPTY_STRING_ARRAY = new String[0];
+
     private static final Xpp3Dom[] EMPTY_DOM_ARRAY = new Xpp3Dom[0];
 
     public static final String CHILDREN_COMBINATION_MODE_ATTRIBUTE = "combine.children";
@@ -94,7 +98,13 @@ public class Xpp3Dom
      */
     public Xpp3Dom( Xpp3Dom src, String name )
     {
-        this( name );
+        this.name = name;
+
+        int childCount = src.getChildCount();
+
+        childList = new ArrayList( childCount );
+        childMap = new HashMap( childCount << 1 );
+
         setValue( src.getValue() );
 
         String[] attributeNames = src.getAttributeNames();
@@ -104,10 +114,9 @@ public class Xpp3Dom
             setAttribute( attributeName, src.getAttribute( attributeName ) );
         }
 
-        Xpp3Dom[] children = src.getChildren();
-        for ( int i = 0; i < children.length; i++ )
+        for ( int i = 0; i < childCount; i++ )
         {
-            addChild( new Xpp3Dom( children[i] ) );
+            addChild( new Xpp3Dom( src.getChild( i ) ) );
         }
     }
 
@@ -140,13 +149,13 @@ public class Xpp3Dom
 
     public String[] getAttributeNames()
     {
-        if ( null == attributes )
+        if ( null == attributes || attributes.isEmpty() )
         {
-            return new String[0];
+            return EMPTY_STRING_ARRAY;
         }
         else
         {
-            return (String[]) attributes.keySet().toArray( new String[0] );
+            return (String[]) attributes.keySet().toArray( new String[attributes.size()] );
         }
     }
 
@@ -199,13 +208,13 @@ public class Xpp3Dom
 
     public Xpp3Dom[] getChildren()
     {
-        if ( null == childList )
+        if ( null == childList || childList.isEmpty() )
         {
             return EMPTY_DOM_ARRAY;
         }
         else
         {
-            return (Xpp3Dom[]) childList.toArray( EMPTY_DOM_ARRAY );
+            return (Xpp3Dom[]) childList.toArray( new Xpp3Dom[childList.size()] );
         }
     }
 
@@ -229,7 +238,7 @@ public class Xpp3Dom
                 }
             }
 
-            return (Xpp3Dom[]) children.toArray( EMPTY_DOM_ARRAY );
+            return (Xpp3Dom[]) children.toArray( new Xpp3Dom[children.size()] );
         }
     }
 
@@ -334,7 +343,7 @@ public class Xpp3Dom
 
         String selfMergeMode = dominant.getAttribute( SELF_COMBINATION_MODE_ATTRIBUTE );
 
-        if ( isNotEmpty( selfMergeMode ) && SELF_COMBINATION_OVERRIDE.equals( selfMergeMode ) )
+        if ( SELF_COMBINATION_OVERRIDE.equals( selfMergeMode ) )
         {
             mergeSelf = false;
         }
@@ -357,50 +366,70 @@ public class Xpp3Dom
                 }
             }
 
-            boolean mergeChildren = true;
-
-            if ( childMergeOverride != null )
+            if ( recessive.getChildCount() > 0 )
             {
-                mergeChildren = childMergeOverride.booleanValue();
-            }
-            else
-            {
-                String childMergeMode = dominant.getAttribute( CHILDREN_COMBINATION_MODE_ATTRIBUTE );
+                boolean mergeChildren = true;
 
-                if ( isNotEmpty( childMergeMode ) && CHILDREN_COMBINATION_APPEND.equals( childMergeMode ) )
+                if ( childMergeOverride != null )
                 {
-                    mergeChildren = false;
-                }
-            }
-
-            Xpp3Dom[] dominantChildren = dominant.getChildren();
-            if ( !mergeChildren )
-            {
-                // remove these now, so we can append them to the recessive list later.
-                dominant.childList.clear();
-            }
-
-            Xpp3Dom[] children = recessive.getChildren();
-            for ( int i = 0; i < children.length; i++ )
-            {
-                Xpp3Dom child = children[i];
-                Xpp3Dom childDom = dominant.getChild( child.getName() );
-                if ( mergeChildren && ( childDom != null ) )
-                {
-                    mergeIntoXpp3Dom( childDom, child, childMergeOverride );
+                    mergeChildren = childMergeOverride.booleanValue();
                 }
                 else
                 {
-                    dominant.addChild( new Xpp3Dom( child ) );
-                }
-            }
+                    String childMergeMode = dominant.getAttribute( CHILDREN_COMBINATION_MODE_ATTRIBUTE );
 
-            if ( !mergeChildren )
-            {
-                // now, re-add these children so they'll be appended to the recessive list.
-                for ( int i = 0; i < dominantChildren.length; i++ )
+                    if ( CHILDREN_COMBINATION_APPEND.equals( childMergeMode ) )
+                    {
+                        mergeChildren = false;
+                    }
+                }
+
+                if ( !mergeChildren )
                 {
-                    dominant.addChild( dominantChildren[i] );
+                    Xpp3Dom[] dominantChildren = dominant.getChildren();
+                    // remove these now, so we can append them to the recessive list later.
+                    dominant.childList.clear();
+
+                    for ( int i = 0, recessiveChildCount = recessive.getChildCount(); i < recessiveChildCount; i++ )
+                    {
+                        Xpp3Dom recessiveChild = recessive.getChild( i );
+                        dominant.addChild( new Xpp3Dom( recessiveChild ) );
+                    }
+
+                    // now, re-add these children so they'll be appended to the recessive list.
+                    for ( int i = 0; i < dominantChildren.length; i++ )
+                    {
+                        dominant.addChild( dominantChildren[i] );
+                    }
+                }
+                else
+                {
+                    Map commonChildren = new HashMap();
+
+                    for ( Iterator it = recessive.childMap.keySet().iterator(); it.hasNext(); )
+                    {
+                        String childName = (String) it.next();
+                        Xpp3Dom[] dominantChildren = dominant.getChildren( childName );
+                        if ( dominantChildren.length > 0 )
+                        {
+                            commonChildren.put( childName, Arrays.asList( dominantChildren ).iterator() );
+                        }
+                    }
+
+                    for ( int i = 0, recessiveChildCount = recessive.getChildCount(); i < recessiveChildCount; i++ )
+                    {
+                        Xpp3Dom recessiveChild = recessive.getChild( i );
+                        Iterator it = (Iterator) commonChildren.get( recessiveChild.getName() );
+                        if ( it == null )
+                        {
+                            dominant.addChild( new Xpp3Dom( recessiveChild ) );
+                        }
+                        else if ( it.hasNext() )
+                        {
+                            Xpp3Dom dominantChild = (Xpp3Dom) it.next();
+                            mergeIntoXpp3Dom( dominantChild, recessiveChild, childMergeOverride );
+                        }
+                    }
                 }
             }
         }
